/* 	Attest.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-feb-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.attest;

import java.awt.GraphicsEnvironment;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

import javax.swing.JOptionPane;

import org.brains2b.attest.gui.AttestView;
import org.brains2b.console.ArgumentException;
import org.brains2b.console.ArgumentMap;
import org.brains2b.console.Main;
import org.brains2b.log.Logger;

public class Attest extends Main {

    public Attest() {
        super();
        init();
    }
    
    public static void main(String[] args) {
        Attest attest = new Attest();
        Logger.println(System.getProperty("os.name"),Logger.DEBUG);
        if ("Mac OS X".equals(System.getProperty("os.name"))) {
        		try {
	        		OsStub macOsx= (OsStub)  Class.forName("org.brains2b.attest.macosx.MacOsXHandler").newInstance();
	        		while (macOsx.getFilename()==null) {
	        			try {
	        				Thread.sleep(300);
	        			} catch (Exception ex) {}
	        		}
	        		attest.setArgument("jar",macOsx.getFilename());
        		} catch (Exception ex) {
        			attest.showException(ex);
        		}
        } else {
        		attest.makeArgumentList(args);
        }
        attest.doIt();
    }
    
    
    private void doIt() {
        try {
            File f = new File(getArgument("jar"));
            JarFile jar = new JarFile(f);
            Manifest mf=jar.getManifest();
            
            if (mf==null) {
            	throw new FileNotFoundException("No manifest in: " + jar.getName());
            }
            
            //If we asked for console or there are no Graphics, print an quit
            if (GraphicsEnvironment.isHeadless() || "true".equals(getArgument("console"))) {
                mf.write(System.out);
                System.exit(0);
            }
            AttestView view =new AttestView();
            view.setManifest(mf, f.getParentFile());
            view.setVisible(true);
        } catch (IOException iex) {
           showException(iex);
           Logger.printException(iex); 
        }
    }
    
    private void init() { 
        ArgumentMap map =new ArgumentMap();
        map.addArgument("jar", true);
        map.addOption("console",'c',false);
        setArgumentMap(map);
    }
    
    protected void handleException(ArgumentException ex) {
        if (GraphicsEnvironment.isHeadless() || "true".equals(getArgument("console")) ) {
            super.handleException(ex);
        } else {
           showException(ex);
           System.exit(1);
        }
    }
    
    private void showException(Exception ex) {
    	if (GraphicsEnvironment.isHeadless() || "true".equals(getArgument("console"))) return;
        JOptionPane.showMessageDialog(null, ex.getMessage(),"@test:error",JOptionPane.ERROR_MESSAGE);
    }
    
}
