/* 	AttestView.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-feb-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.attest.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.MouseInfo;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

public class AttestView extends JFrame implements ActionListener,MouseMotionListener, MouseListener {
    
    private JTextPane m_textPane;
    private HashMap m_children;
    private File m_path;
    private Manifest m_manifest;
    private Stack m_last;
    private JButton m_back;
    
    public AttestView() {
        super();
        init();
        m_last = new Stack();
    }
    
    private void init() {
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setResizable(false);
        setTitle("@test 0.10.1");
        Dimension d= getToolkit().getScreenSize();
        d.height=d.height*30/100;
        d.width=d.width*30/100;
        //JDK 1.5 dependant code
        setLocation(MouseInfo.getPointerInfo().getLocation());
        //
        setSize(d);
        JPanel main = new JPanel(new BorderLayout());
        setContentPane(main);
        main.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        
        m_textPane = new JTextPane();
        m_textPane.setEditable(false);
        m_textPane.addMouseMotionListener(this);
        m_textPane.addMouseListener(this);
        main.add(new JScrollPane(m_textPane),BorderLayout.CENTER);
        
        StyledDocument doc = m_textPane.getStyledDocument();
        Style dflt= StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
        Style notfound=doc.addStyle("notfound", dflt);
        StyleConstants.setForeground(notfound, Color.red);
        StyleConstants.setStrikeThrough(notfound, true);
            
        Style link= doc.addStyle("link", doc.getStyle(StyleContext.DEFAULT_STYLE));
        StyleConstants.setForeground(link, Color.blue);
        StyleConstants.setUnderline(link, true);
        
        m_back= new JButton("<<");
        m_back.setActionCommand("Back");
        m_back.addActionListener(this);
        m_back.setVisible(false);
        
        JButton b= new JButton("Ok");
        b.setActionCommand("Close");
        b.addActionListener(this);
        JPanel p= new JPanel(new BorderLayout());
        p.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
        p.add(m_back,BorderLayout.WEST);
        p.add(b,BorderLayout.EAST);
        main.add(p,BorderLayout.SOUTH);    
    }
    
    public void setManifest(Manifest mf, File path) {
        m_children= new HashMap();
        m_manifest=mf;
        m_path=path;
        ByteArrayOutputStream baos= new ByteArrayOutputStream(1024);
        String text=null;
        try {
            mf.write(baos);
            baos.flush();
            text=new String(baos.toByteArray(),"ISO8859_1");
            text= text.replaceAll("\r\n ", ""); //Clean it up a bit
            m_textPane.setText(text);
            
        } catch (Exception ex) {
            showException(ex);
            return;
        } finally {
            try {
                baos.close();
            } catch (IOException iex) {
            }
        }
        
        StyledDocument doc = m_textPane.getStyledDocument();
            
        doc.setCharacterAttributes(0, doc.getLength(), doc.getStyle(StyleContext.DEFAULT_STYLE), true);
        m_textPane.setCaretPosition(0);
        try {
            String as = (String) mf.getMainAttributes().get(Attributes.Name.CLASS_PATH);
            if (as!=null) {
                int cidx= text.indexOf("Class-Path");
                StringTokenizer st= new StringTokenizer(as);
                while(st.hasMoreTokens()) {
                    String jarName = st.nextToken();
                    File fj =  new File(path,jarName);
                    
                    int idx= text.indexOf(jarName,cidx)-1;
                    if (fj.exists()) {
                        JarFile jar = new JarFile(fj);
                        Manifest jMf= jar.getManifest();
                        jar.close();
                        if (jMf!=null) {
                            doc.setCharacterAttributes(idx, jarName.length(), doc.getStyle("link"), false);
                            m_children.put(jarName, jMf);
                        }
                    } else {
                        doc.setCharacterAttributes(idx, jarName.length(), doc.getStyle("notfound"), false);
                    }
                }
            }
        } catch(Exception blex) {
            showException(blex);
            return;
        }
        
        
    }
    
    public void actionPerformed(ActionEvent e) {
        if ("Close".equals(e.getActionCommand())) {
            dispose();
        } else if ("Back".equals(e.getActionCommand())) {
            setManifest((Manifest) m_last.pop(), m_path);
            if (m_last.isEmpty()) {
                m_back.setVisible(false);
            }
        }
    }
    
    public void mouseMoved(MouseEvent e) {
        int pos= m_textPane.viewToModel(e.getPoint());
        Element el= m_textPane.getStyledDocument().getCharacterElement(pos);
        if (el.getAttributes().containsAttribute(SimpleAttributeSet.NameAttribute,"link")) {
            m_textPane.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        } else {
            m_textPane.setCursor(Cursor.getDefaultCursor());
        }
    }
    
    public void mouseClicked(MouseEvent e) {
        int pos= m_textPane.viewToModel(e.getPoint());
        Element el= m_textPane.getStyledDocument().getCharacterElement(pos);
        if (el.getAttributes().containsAttribute(SimpleAttributeSet.NameAttribute,"link")) {
            try {
                String s=m_textPane.getStyledDocument().getText(el.getStartOffset(), el.getEndOffset()-el.getStartOffset());
                Manifest mf= (Manifest) m_children.get(s);
                if (mf!=null) {
                    m_last.push(m_manifest);
                    m_back.setVisible(true);
                    setManifest(mf, m_path);
                }
                    
            } catch (BadLocationException blex) {
                showException(blex);
            }
        }
    }
    
    private void showException(Exception ex) {
        JOptionPane.showMessageDialog(this, ex.getMessage(),"@test:error",JOptionPane.ERROR_MESSAGE);
    }
    
    public void mouseDragged(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }
 
    
}
